// Software-PWM by PoWl - 25.10.08
// webmaster@the-powl.de
//
// Anpassung N. Turianskyj 03.07.2011
// fr SEDU-Board bzw. als allgemeine Bibliothek
// hier sind keine nderungen notwendig!


#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdbool.h>

#include "SoftPWM1p.h"



uint8_t pwm_pin[CHANNELS];
uint8_t pwm_mask_A[CHANNELS + 1];
uint8_t pwm_timing_A[CHANNELS + 1];

uint8_t pwm_mask_B[CHANNELS + 1];
uint8_t pwm_timing_B[CHANNELS + 1];

uint8_t *ptr_PORT_main = &pwm_mask_A;
uint8_t *ptr_timing_main = &pwm_timing_A;

uint8_t *ptr_PORT_isr = &pwm_mask_B;
uint8_t *ptr_timing_isr = &pwm_timing_B;

uint8_t pwm_value[CHANNELS];
uint8_t pwm_queue[CHANNELS];

volatile uint8_t pwm_cycle;
volatile bool pwm_change;




void pwm_update(void)
{
  pwm_change = 0;

  // Sortieren mit InsertSort

  for(uint8_t i=1; i<CHANNELS; i++)
  {
    uint8_t j = i;

    while(j && pwm_value[pwm_queue[j - 1]] > pwm_value[pwm_queue[j]])
    {
      uint8_t temp = pwm_queue[j - 1];
      pwm_queue[j - 1] = pwm_queue[j];
      pwm_queue[j] = temp;

      j--;
    }
  }

  // Maske generieren

  for(uint8_t i=0; i<(CHANNELS + 1); i++)
  {
    ptr_PORT_main[i] = 0;
    ptr_timing_main[i] = 0;
  }

  uint8_t j = 0;

  for(uint8_t i=0; i<CHANNELS; i++)
  {
    // Sonderfall "ganz AUS" ausschlieen:
    //  - Pins garnicht erst anschalten
    //  - Pins die vorher "ganz AN" waren mssen trotzdem noch deaktiviert werden, deshalb nur folgendes in der if
    if(pwm_value[pwm_queue[i]] != 0)
    {
      ptr_PORT_main[0] |= (1 << pwm_pin[pwm_queue[i]]);
    }

    // Sonderfall "ganz AN" ausschlieen:
    //  - Nicht in Timing-Tabelle bernehmen da sonst die Pointer niemals getauscht werden
    //  - Pins die "ganz AN" sind mssen nicht abgeschaltet werden
    if(pwm_value[pwm_queue[i]] != 255)
    {
      ptr_PORT_main[j + 1] |= (1 << pwm_pin[pwm_queue[i]]);

      ptr_timing_main[j] = pwm_value[pwm_queue[i]];
    }

    if(pwm_value[pwm_queue[i]] != pwm_value[pwm_queue[i + 1]])
    {
      j++;
    }
  }

  /*
  // Der ISR etwas Arbeit abnehmen
  for(uint8_t i=1; i<(CHANNELS + 1); i++)
  {
    ptr_PORT_main[i] = ~ptr_PORT_main[i];
  }
  */

  pwm_change = 1;
}




ISR(TIMER1_COMPA_vect)
{
  pwm_cycle = 0;

  PWM_PORT |= ptr_PORT_isr[0];
  OCR1B = ptr_timing_isr[0];
}



ISR(TIMER1_COMPB_vect)
{
  pwm_cycle++;

  PWM_PORT &= ~ptr_PORT_isr[pwm_cycle];

  if(ptr_timing_isr[pwm_cycle] != 0)
  {
    OCR1B = ptr_timing_isr[pwm_cycle];
  }
  else if(pwm_change)
  {
    pwm_change = 0;

    uint8_t *temp_ptr;

    temp_ptr = ptr_PORT_main;
    ptr_PORT_main = ptr_PORT_isr;
    ptr_PORT_isr = temp_ptr;

    temp_ptr = ptr_timing_main;
    ptr_timing_main = ptr_timing_isr;
    ptr_timing_isr = temp_ptr;
  }
}
